/**
 * sequential mandelbrot
 */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define N (512)
#define MAX_ITERS (10000)

int image[N*N];

#define get_pixel(x,y) (image[(y)*N + (x)])
#define set_pixel(x,y,v) image[(y)*N + (x)] = (v)

//
// IO
//
void dump_file() {
  FILE *fp;
  int x,y;
  
  fp = fopen("out.dat","w");
  
  for (y=0; y < N; y++) {
      for (x=0; x < N; x++) {
        fprintf(fp,"%d ", get_pixel(x,y));
      }
      fprintf(fp,"\n");
  }
  
  fclose(fp);
}

//
// iterate z_n+1 = z_n^2 + c
//
// for the complex number c = a+bi
//
int count_iterations(double a, double b) {
  int n;
  double zr = 0.0, zi = 0.0;
  double tzr, tzi;

  const double t = 4.0;
  const int max_iters = MAX_ITERS;

  n = 0;
  while (n < max_iters && sqrt(zr*zr + zi*zi) < t) {
    tzr = (zr*zr - zi*zi) + a;
    tzi = (zr*zi + zr*zi) + b;
    zr = tzr;
    zi = tzi;
    n = n+1;
  }

  return n;
}

int main() {
  double a,   b,  c,  d;
  double ta, tb, tc, td;
  double dx, dy;

  int nTiles = 128;
  int i, j;
  
  a = -2.0; b = -1.25; 
  c =  0.5; d =  1.25; 

  dx = (c - a)/N;
  dy = (d - b)/N;

  for (i=0; i < N; i++) {
     for (j=0; j < N; j++) {
        int count = count_iterations(a + i*dx, b + j*dy);
        set_pixel(i, j, count);
     }
  }

  dump_file();
  return 0;
}
